package com.develop.setting.init.ui

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import com.alibaba.android.arouter.facade.annotation.Route
import com.azhon.appupdate.listener.OnDownloadListener
import com.blankj.utilcode.util.AppUtils
import com.develop.food.base.api.FoodRepository
import com.develop.food.base.http.observeStatus
import com.develop.food.base.router.Screens
import com.develop.food.base.ui.BaseActivity
import com.develop.food.base.utils.GlobalToast
import com.develop.food.base.utils.UpdateUtil
import com.develop.food.base.widgets.CancelConfirmDialog
import com.develop.food.base.widgets.CommonDialog
import com.develop.setting.R
import com.develop.setting.databinding.ActivityAboutBinding
import java.io.File

@Route(path = Screens.Setting.ABOUT)
class AboutActivity : BaseActivity<ActivityAboutBinding>() {
    override fun createViewBinding(inflater: LayoutInflater): ActivityAboutBinding {
        return ActivityAboutBinding.inflate(layoutInflater)
    }

    private var updateApkUrl = ""

    private val cancelConfirmDialog by lazy {
        CancelConfirmDialog()
    }

    private val commonDialog by lazy {
        CommonDialog()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding.topBar.setTopBarTitle(getString(R.string.About))
        binding.topBar.listenBackClick {
            finish()
        }


        commonDialog.msg = "系统更新中，请稍候～"
        commonDialog.hasOKBtn = false
        cancelConfirmDialog.title = "是否更新当前版本？"
        cancelConfirmDialog.onDialogClickListener =
            object : CancelConfirmDialog.OnDialogClickListener {
                override fun onConfirm() {
                    commonDialog.show(supportFragmentManager, "commonDialog")
                    UpdateUtil.updateApp(this@AboutActivity, updateApkUrl,object :OnDownloadListener{
                        override fun cancel() {

                        }

                        override fun done(apk: File) {
                            GlobalToast.showToast("下载完成")
                        }

                        override fun downloading(max: Int, progress: Int) {
                            runOnUiThread {
                                commonDialog.updateProgress("${progress}%")
                            }
                        }

                        override fun error(e: Throwable) {
                            GlobalToast.showToast("下载失败")
                        }

                        override fun start() {

                        }

                    })
                }

                override fun onCancel() {

                }
            }

        binding.tvUpdate.setOnClickListener {
            cancelConfirmDialog.show(supportFragmentManager, "cancelConfirmDialog")
        }

        binding.tvAppName.text = AppUtils.getAppName()
        binding.tvAppVersion.text = "v${AppUtils.getAppVersionName()}"

    }

    override fun onResume() {
        super.onResume()
        FoodRepository.devInfo().observeStatus(onSuccess = {
            updateApkUrl = it?.apkUrl ?: ""
            if (it?.apkUpdate == true) {
                binding.tvUpdate.visibility = View.VISIBLE
            } else {
                binding.tvUpdate.visibility = View.INVISIBLE
            }
        })
    }

    override fun onDestroy() {
        super.onDestroy()
        if (commonDialog.dialog?.isShowing == true) {
            commonDialog.dismiss()
        }
        if (cancelConfirmDialog.dialog?.isShowing == true) {
            cancelConfirmDialog.dismiss()
        }
    }
}