package com.develop.setting.init.ui


import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.RelativeLayout
import com.alibaba.android.arouter.facade.annotation.Route
import com.blankj.utilcode.util.LanguageUtils
import com.develop.food.base.common.*
import com.develop.food.base.data.DataFactory
import com.develop.food.base.event.FinishAtyEvent
import com.develop.food.base.router.Screens
import com.develop.food.base.router.navigateTo
import com.develop.food.base.ui.BaseActivity
import com.develop.food.base.utils.MmkvUtils
import com.develop.food.base.utils.ResDimension
import com.develop.setting.R
import com.develop.setting.databinding.ActivityLanguageBinding
import com.develop.setting.init.adapter.LanguageAdapter
import org.greenrobot.eventbus.EventBus
import org.greenrobot.eventbus.Subscribe
import java.util.*

@Route(path = Screens.Setting.LANGUAGE)
class LanguageActivity : BaseActivity<ActivityLanguageBinding>() {
    private var languageAdapter = LanguageAdapter()
    private var selectPosition = -1
    override fun createViewBinding(inflater: LayoutInflater): ActivityLanguageBinding {
        return ActivityLanguageBinding.inflate(layoutInflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        EventBus.getDefault().register(this)
        val isFromSetting = intent.extras?.getBoolean(LANGUAGE_FROM_SETTING) ?: false
        if (isFromSetting) {
            binding.ivBanner.visibility = View.GONE
            binding.topBar.visibility = View.VISIBLE
            binding.tvLanguage.visibility = View.GONE
            binding.topBar.setTopBarTitle(getString(R.string.Language))
            val params = binding.languageRecycle.layoutParams as RelativeLayout.LayoutParams
            params.topMargin = ResDimension(R.dimen.convert_251px).toInt()
            binding.languageRecycle.layoutParams = params
            binding.topBar.listenBackClick {
                finish()
            }
        }
        languageAdapter.setList(DataFactory.createLanguageData(resources))
        binding.languageRecycle.apply {
            adapter = languageAdapter
            selectPosition = getSelectedPos()
        }

        languageAdapter.setOnItemClickListener { adapter, view, position ->
            languageAdapter.updateSelectedPosition(position)
            this.selectPosition = position
            updateLanguage()
        }

        binding.tvNext.setOnClickListener {
            updateLanguage()
            if (isFromSetting) {
                finish()
            } else {
                navigateTo(Screens.Setting.POLICY)
            }
        }
    }

    private fun updateLanguage() {
        when (selectPosition) {
            0, -1 -> {
                MmkvUtils.save(CURRENT_LANGUAGE, EN)
                LanguageUtils.applyLanguage(Locale.ENGLISH)
            }
            1 -> {
                MmkvUtils.save(CURRENT_LANGUAGE, ZH)
                LanguageUtils.applyLanguage(Locale.CHINESE)
            }
            2 -> {
                MmkvUtils.save(CURRENT_LANGUAGE, FR)
                LanguageUtils.applyLanguage(Locale.FRANCE)
            }
            3 -> {
                MmkvUtils.save(CURRENT_LANGUAGE, JA)
                LanguageUtils.applyLanguage(Locale.JAPAN)
            }
        }
    }

    private fun getSelectedPos(): Int {
        return when (MmkvUtils.getString(CURRENT_LANGUAGE)) {
            EN -> {
                languageAdapter.updateSelectedPosition(0)
                0
            }
            ZH -> {
                languageAdapter.updateSelectedPosition(1)
                1
            }

            FR -> {
                languageAdapter.updateSelectedPosition(2)
                2
            }

            JA -> {
                languageAdapter.updateSelectedPosition(3)
                3
            }
            else -> {
                languageAdapter.updateSelectedPosition(0)
                0
            }
        }
    }

    @Subscribe
    fun onFinishAtyEvent(event: FinishAtyEvent) {
        finish()
    }

    override fun onDestroy() {
        super.onDestroy()
        EventBus.getDefault().unregister(this)
    }
}