package com.develop.setting.init.ui

import android.graphics.Rect
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.facade.annotation.Route
import com.blankj.utilcode.util.LogUtils
import com.blankj.utilcode.util.NetworkUtils
import com.develop.food.base.common.WIFI_FROM_MAIN_SETTING
import com.develop.food.base.event.FinishAtyEvent
import com.develop.food.base.manager.wifi.AccessPoint
import com.develop.food.base.manager.wifi.WifiReceiver
import com.develop.food.base.manager.wifi.WifiUtils
import com.develop.food.base.router.Screens
import com.develop.food.base.router.navigateTo
import com.develop.food.base.ui.BaseActivity
import com.develop.food.base.utils.ResDimension
import com.develop.setting.R
import com.develop.setting.databinding.ActivityWifiListBinding
import com.develop.setting.init.adapter.WifiListAdapter
import com.yanzhenjie.permission.AndPermission
import com.yanzhenjie.permission.runtime.Permission
import org.greenrobot.eventbus.EventBus

@Route(path = Screens.Setting.WIFI)
class WifiListActivity : BaseActivity<ActivityWifiListBinding>(), WifiReceiver.WifiStateListener,
    NetworkUtils.OnNetworkStatusChangedListener {
    private var wifiListAdapter = WifiListAdapter()
    private var isFromMainSetting = false
    private var accessPointList = mutableListOf<AccessPoint>()
    override fun createViewBinding(inflater: LayoutInflater): ActivityWifiListBinding {
        return ActivityWifiListBinding.inflate(layoutInflater)
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        WifiUtils.getInstance().init(this)
        NetworkUtils.registerNetworkStatusChangedListener(this)
        isFromMainSetting = intent.extras?.getBoolean(WIFI_FROM_MAIN_SETTING) ?: false

        if (isFromMainSetting) {
            binding.tvSkin.visibility = View.GONE
        }

        binding.topBar.listenBackClick {
            finish()
        }
        binding.topBar.setTopBarTitle(getString(R.string.wifi))


        wifiListAdapter.apply {
            onOkClickListener = object : WifiListAdapter.OnOkClickListener {
                override fun onOkClick(accessPoint: AccessPoint) {
                    WifiUtils.getInstance()
                        .connectWifi(this@WifiListActivity, accessPoint.ssid, accessPoint.password)
                }
            }
            setOnItemClickListener { adapter, view, position ->
                val accessPoint = accessPointList[position]
                if (!accessPoint.isSecured) {
                    WifiUtils.getInstance().connectWifi(accessPoint)
                } else {
                    updateSelectedPosition(position)
                }
            }
        }

        binding.wifiRecycler.apply {
            adapter = wifiListAdapter
            addItemDecoration(object : RecyclerView.ItemDecoration() {
                override fun getItemOffsets(
                    outRect: Rect,
                    view: View,
                    parent: RecyclerView,
                    state: RecyclerView.State
                ) {
                    super.getItemOffsets(outRect, view, parent, state)
                    val pos = parent.getChildAdapterPosition(view)
                    if (pos == 0) {
                        outRect.top = ResDimension(R.dimen.convert_30px).toInt()
                    }
                    outRect.left = ResDimension(R.dimen.convert_45px).toInt()
                    outRect.right = ResDimension(R.dimen.convert_45px).toInt()
                    outRect.bottom = ResDimension(R.dimen.convert_50px).toInt()
                }
            })
        }


        binding.tvSkin.setOnClickListener {
            if (NetworkUtils.isConnected()) {
                navigateTo(Screens.Auth.MEMBER)
            } else {
                navigateTo(Screens.Main.ENTRANCE_CHOSEN)
            }
            EventBus.getDefault().post(FinishAtyEvent())
            finish()
        }

        binding.tvNext.setOnClickListener {
            EventBus.getDefault().post(FinishAtyEvent())
            if (isFromMainSetting) {
                finish()
            } else {
                navigateTo(Screens.Auth.MEMBER)
            }

        }
    }

    override fun onDestroy() {
        super.onDestroy()
        if (WifiUtils.getInstance().isRegisterWifiBroadcast) {
            WifiUtils.getInstance().unregisterWifiBroadcast()
            WifiUtils.getInstance().release()
        }
    }

    override fun onResume() {
        super.onResume()
        if (!WifiUtils.getInstance().isRegisterWifiBroadcast) {
            WifiUtils.getInstance().registerWifiBroadcast(this)
        }

        AndPermission.with(this).runtime().permission(
            Permission.ACCESS_FINE_LOCATION,
            Permission.ACCESS_BACKGROUND_LOCATION, Permission.ACCESS_COARSE_LOCATION
        )
            .onGranted {
                WifiUtils.getInstance().openWifi()
            }.start()
    }

    override fun onWifiOpen() {
        LogUtils.d("open")
    }

    override fun updateAccessPoints(accessPoints: MutableList<AccessPoint>) {
        LogUtils.d("updateAccessPoints %d",accessPoints.size)
        accessPointList.clear()
        accessPointList.addAll(accessPoints)
        wifiListAdapter.setList(accessPointList)
    }

    override fun onWifiClose() {
        LogUtils.d("onWifiClose")
    }

    override fun onHotpotOpen() {

    }

    override fun onHotpotOpenError() {

    }

    override fun onHotpotClose() {

    }

    override fun onDisconnected() {
        LogUtils.d("onDisconnected")
    }

    override fun onConnected(networkType: NetworkUtils.NetworkType) {
        LogUtils.d("onConnected")
    }
}