package com.develop.step

import android.annotation.SuppressLint
import android.os.Bundle
import android.util.Log
import android.view.KeyEvent
import com.alibaba.android.arouter.facade.annotation.Route
import com.bumptech.glide.Glide
import com.bumptech.glide.load.resource.drawable.DrawableTransitionOptions
import com.develop.food.base.common.PRESS_DOWN_KEY_CODE
import com.develop.food.base.common.TURN_DOWN_KEY_CODE
import com.develop.food.base.common.TURN_UP_KEY_CODE
import com.develop.food.base.data.model.CookSettingType
import com.develop.food.base.event.CookStepEvent
import com.develop.food.base.manager.FloatWindowManager
import com.develop.food.base.repo.CURRENT_USER_ID
import com.develop.food.base.repo.FoodDataProvider
import com.develop.food.base.repo.entity.UserHistoryRecipes
import com.develop.food.base.router.Screens
import com.develop.food.base.utils.*
import com.develop.food.base.widgets.CancelConfirmDialog
import com.develop.food.base.widgets.TimePickerView
import com.develop.step.viewmodel.CookStepStatus
import com.develop.step.viewmodel.CookStepUiData
import com.kuyuntech.cofarcooking.device.sdk.constant.core.DevStatus
import com.kuyuntech.cofarcooking.device.sdk.constant.core.HeatModes
import com.kuyuntech.cofarcooking.device.sdk.constant.core.MotorDirections
import com.kuyuntech.cofarcooking.device.sdk.eventbus.core.DevInfo
import com.kuyuntech.cofarcooking.device.sdk.util.core.CofarSDK
import org.greenrobot.eventbus.EventBus
import java.io.File

@Route(path = Screens.Cook.COOK_STEP2)
class CookStepActivity2 : CookStepToolActivity() {

    private var timeArray = IntArray(3)
    private val audioUtil = MusicBackPlayerUtil()
    private var audioMute = false
    private var pendingCookStep = false
    private var isHistory = false
    private var mSettingInstId = ""
    private var mRunningInstId = ""
    private var potCloverStatus = 1 //锅盖是否盖上 0 是 1 否
    private var coverPath: Any? = null
    private var isFromOverWrite = false

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding.controller.clSetTime.onTimePickerCallback =
            object : TimePickerView.OnTimePickerCallback {
                override fun onTimePicker(
                    hours: Int,
                    minute: Int,
                    second: Int,
                    time: String,
                    setByUser: Boolean
                ) {
                    if (setByUser) {
                        viewModel.stepUiData.doingModify = true
                        val totalTime = second + (minute + hours * 60) * 60
                        viewModel.displayStep()?.uiData?.targetTime = totalTime
                        binding.controller.tvCookingTimeTarget.updateText("--${time}--")
                        CofarSDK.cfgTime(totalTime.toShort())
                    }
                }

                override fun onTimePickerTouchFirst() {
                    viewModel.stepUiData.doingModify = true
                }
            }
        binding.viewAlarm.setOnClickListener {
            audioMute = !audioMute
            if (audioMute) {
                audioUtil.stopBgSound()
                binding.viewAlarm.setBackgroundResource(R.drawable.ic_audio_mute)
            } else {
                binding.viewAlarm.setBackgroundResource(R.drawable.ic_alarm)
            }
        }
        initData()
    }

    private fun initData() {
        viewModel.queryRecipeCookStep(recipeNumber ?: "", currentStepIndex)
        viewModel.stepDisplay.observe(this) {
            coverPath = FoodDataProvider.getImagePath(it.source.photoVideoFilePath)
            if (runningState == 0) {
                CofarSDK.changeMode(
                    "$recipeNumber:$currentStepIndex",
                    CofarSDK.devMode(it.workMode)
                )
            }
            if (it.isWeightMode()) {
                changeWeightPanel()
                CofarSDK.startWeight()
            } else if (it.isTurboMode()) {
                changeTurboPanel()
            } else {
                CofarSDK.stoptWeight()
                changeTempSettingPanel()
            }
            val stepCount = viewModel.getStepCount()
            val stepIndex = viewModel.allSteps.indexOf(it)
            //通知当前步骤信息给浮窗
            EventBus.getDefault()
                .post(
                    CookStepEvent(
                        recipeNumber ?: "",
                        coverPath as String,
                        stepIndex
                    )
                )
            cookProgressDrawable.setStepState(stepIndex, stepCount)
            binding.tvStepName.text = it.source.description
            binding.tvTitleStep.text = it.source.description
            if (stepCount == 1) {
                binding.ivPrevStep.setGone()
                binding.ivNextStep.setGone()
            } else if (viewModel.isFirstStep(it)) {
                binding.ivPrevStep.setGone()
                binding.ivNextStep.setVisible()
            } else if (viewModel.isFinalStep(it)) {
                binding.ivPrevStep.setVisible()
                binding.ivNextStep.setGone()
            } else {
                binding.ivPrevStep.setVisible()
                binding.ivNextStep.setVisible()
            }
            Glide.with(binding.ivCookBg)
                .load(FoodDataProvider.getImagePath(it.source.photoVideoFilePath))
                .transition(DrawableTransitionOptions.withCrossFade())
                .error(R.drawable.bg_cook_session)
                .into(binding.ivCookBg)
            updateUiSetting(it)
            playStepAudio(it.source.audioFilePath)
            tryConsumePendingCookStep(it)
        }

        backRequestDialog.apply {
            title = "Keep cooking in the background?"
            onDialogClickListener = object : CancelConfirmDialog.OnDialogClickListener {
                override fun onConfirm() {
                    val cookStepEvent = CookStepEvent(
                        recipeNumber ?: "",
                        coverPath as String,
                        currentStepIndex,
                        isMode = false
                    )
                    FloatWindowManager.showStepFlowWindow(cookStepEvent)
                    finish()
                }

                override fun onCancel() {
                    FloatWindowManager.hideStepFlowWindow()
                    CofarSDK.stop()
                    finish()
                }
            }
        }

        overrideModeDialog.apply {
            title =
                "Overwrite current cooking process?\n By overwriting,your current progress will got lost."
            onDialogClickListener = object : CancelConfirmDialog.OnDialogClickListener {
                override fun onConfirm() {
                    isFromOverWrite = true
                    viewModel.displayStep()?.let {
                        viewModel.setTargetCookingStep(it)
                        configDataAndStart(it.uiData)
                    }
                    val cookStepEvent = CookStepEvent(
                        recipeNumber ?: "",
                        coverPath as String,
                        currentStepIndex,
                        isMode = false
                    )
                    FloatWindowManager.showStepFlowWindow(cookStepEvent)
                    recordRecipes()
                }

                override fun onCancel() {

                }
            }
        }

    }

    private fun playStepAudio(audioPath: String?) {
        audioUtil.stopBgSound()
        if (audioMute) {
            return
        }
        if (!audioPath.isNullOrEmpty()) {
            val filePath = FoodDataProvider.getResourcePath(audioPath)
            if (File(filePath).exists()) {
                audioUtil.playBgSound(filePath)
            }
        }
    }

    override fun clickNextStep() {
        if (viewModel.isCurrentOnCookingStep()
            && viewModel.cookingStep?.isManualStep() == true
        ) {
            // 当前为手动步骤, 下一个步骤切换会设置为当前烹饪步骤
            pendingCookStep = true
        }
        viewModel.nextStep()
    }

    override fun clickPrevStep() {
        if (viewModel.isCurrentOnCookingStep() && viewModel.cookingStep?.isManualStep() == true
            || runningState == DevStatus.STOP.toInt()
        ) {
            // 当前为手动步骤, 上一个步骤切换会设置为当前烹饪步骤
            pendingCookStep = true
        }
        viewModel.prevStep()
    }

    /**
     * 前一个步骤为称重步骤, 切换到左/右步骤后自动设定为当前步骤, 有自动开始字段则自动开始烹饪
     */
    private fun tryConsumePendingCookStep(status: CookStepStatus) {
        if (!pendingCookStep) {
            return
        }
        pendingCookStep = false
        viewModel.setTargetCookingStep(status)
//        if (status.source.autoStart == "1") {
//            configDataAndStart(status.uiData)
//        }
    }

    /**
     * BUTTON_TAG_START
     * BUTTON_TAG_PAUSE
     * BUTTON_TAG_RESUME
     * BUTTON_TAG_CONFIRM
     */
    override fun clickFirstButton(tag: String) {
        handleButtonState(tag)
    }

    /**
     * BUTTON_TAG_CANCEL
     * BUTTON_TAG_STOP
     * BUTTON_TAG_RESET
     */
    override fun clickSecondButton(tag: String) {
        handleButtonState(tag)
    }

    private fun handleButtonState(tag: String) {
        if (tag == BUTTON_TAG_START) {
            if (potCloverStatus == 1) {
                showPotCloverDialog()
            } else if (mRunningInstId != mSettingInstId) {
                overrideModeDialog.show(supportFragmentManager, "overrideModeDialog")
            } else {
                viewModel.displayStep()?.let {
                    viewModel.setTargetCookingStep(it)
                    configDataAndStart(it.uiData)
                }
                recordRecipes()
            }
        } else if (tag == BUTTON_TAG_PAUSE) {
            CofarSDK.pause()
        } else if (tag == BUTTON_TAG_RESUME) {
            CofarSDK.resume()
        } else if (tag == BUTTON_TAG_CONFIRM) {
            CofarSDK.confirm()
            binding.controller.btnStart.setGone()
            binding.controller.btnReset.setGone()
        } else if (tag == BUTTON_TAG_STOP) {
            CofarSDK.stop()
        } else if (tag == BUTTON_TAG_RESET) {
            val current = viewModel.displayStep() ?: return
            val stepIndex = viewModel.allSteps.indexOf(current)
            val uiCopy = viewModel.dataCopy.getOrNull(stepIndex)
            uiCopy?.let {
                current.uiData.copyFrom(it)
                updateUiSetting(current)
            }
        } else if (tag == BUTTON_TAG_CANCEL) {
            CofarSDK.cancel()
            binding.controller.btnStart.setGone()
            binding.controller.btnReset.setGone()
        }
    }

    override fun clickWeightTare() {
        CofarSDK.clearWeight()
    }

    override fun onUserChangeTemp(value: Int) {
        viewModel.displayStep()?.uiData?.targetTemp = value
        binding.controller.tvCookingTempTarget.updateText("--${value}°C--")
        binding.controller.tvRingTempText.updateText("${value}°C")
        CofarSDK.cfgHeat(value.toShort(), HeatModes.PU_TONG)
    }

    override fun onUserChangeSpeed(value: Int) {
        viewModel.displayStep()?.uiData?.currentSpeed = value
        binding.controller.tvSetSpeed.updateText(value.toString())
        binding.controller.tvSpeedText.updateText(value.toString())
        CofarSDK.cfgMotorGear(value.toByte())
    }

    override fun onSettingDirection(value: Int) {
        viewModel.displayStep()?.uiData?.direction = value
        if (value == 0) {
            CofarSDK.cfgMotorDirection(MotorDirections.FORWARD)
        } else {
            CofarSDK.cfgMotorDirection(MotorDirections.REVERSE)
        }
    }

    override fun onDevStateChange(devInfo: DevInfo) {
        if (BuildConfig.DEBUG) {
            Log.d("dddddd", devInfo.toJson())
        }
        viewModel.cookingStep?.let {
            potCloverStatus = devInfo.potCloverStatus.toInt()
            mRunningInstId = devInfo.runningInstId
            mSettingInstId = devInfo.settingInstId
            it.uiData.targetTemp = devInfo.targetTemp.toInt()
            it.uiData.targetTime = devInfo.targetTime.toInt()
            it.uiData.currentSpeed = devInfo.motorGear.toInt()
            it.uiData.weightNum = devInfo.weight.toInt()
            it.uiData.direction = devInfo.motorDirection.toInt()
            it.uiData.runningStatus = devInfo.status
            it.uiData.currentTemp = devInfo.temp.toInt()
            it.uiData.remainTime = devInfo.remainTime.toInt()
            it.uiData.targetTimeBuffer = devInfo.targetTimeBuffer.toInt()
            it.uiData.isTimeChange = devInfo.mode.isTimeChange
            it.uiData.isTempChange = devInfo.mode.isTempChange
            it.uiData.isMotorDirectionChange = devInfo.mode.isMotorDirectionChange
            it.uiData.isMotorGearChange = devInfo.mode.isMotorGearChange
            if (!it.hasStarted && devInfo.status == DevStatus.RUNNING.toInt()) {
                // 标记这个步骤曾经开始过了
                it.hasStarted = true
            }
            if (it.hasStarted && !it.hasComplete && devInfo.status == DevStatus.STOP.toInt()) {
                // 这个步骤目前已经结束了
                it.hasComplete = true
                notifyStepComplete(it)
            }
        }
        if (viewModel.isCurrentOnCookingStep()) {
            viewModel.cookingStep?.let {
                updateUiSetting(it)
            }
        }
        // 重量设置
        viewModel.cookingStep?.apply {
            binding.tvWeightNum.updateText("${uiData.weightNum}g")
            binding.weightView.setWeightNumber(uiData.weightNum.toFloat())
        }
    }


    private fun showPotCloverDialog() {
        val warnDialog = supportFragmentManager.findFragmentByTag("WarningDialog")
        if (warnDialog == null) {
            supportFragmentManager
                .beginTransaction()
                .add(PotCoverUnlockedDialog(), "WarningDialog")
                .commitNowAllowingStateLoss()
        }
    }

    private fun notifyStepComplete(cookStep: CookStepStatus) {
        val dialog = CookStepCompleteDialog {
            val nextStep = viewModel.getNextStep(
                cookStep
            ) ?: return@CookStepCompleteDialog
            if (nextStep.source.autoStart != "1") {
                return@CookStepCompleteDialog
            }
            // 自动切换到下一步骤
            viewModel.setTargetCookingStep(nextStep)
        }
        supportFragmentManager
            .beginTransaction()
            .add(dialog, "complete")
            .commitAllowingStateLoss()
    }

    private fun configDataAndStart(uiData: CookStepUiData) {
        CofarSDK.cfgTime(uiData.targetTime.toShort())
        CofarSDK.cfgMotorDirection(
            if (uiData.direction == 0) {
                MotorDirections.FORWARD
            } else {
                MotorDirections.FORWARD
            }
        )
        CofarSDK.cfgMotorGear(uiData.currentSpeed.toByte())
        CofarSDK.cfgHeat(uiData.targetTemp.toShort(), HeatModes.PU_TONG)
        CofarSDK.cancel()

        if (isFromOverWrite) {
            //覆盖调用
            CofarSDK.startRunning()
            isFromOverWrite = false
        } else {
            CofarSDK.start()
        }
        //通知当前运行的菜谱的ID和当前步骤索引
        CofarSDK.startRecipe(recipeNumber, viewModel.stepIndex.toString())
    }

    override fun dispatchKeyEvent(event: KeyEvent): Boolean {
        if (event.action != KeyEvent.ACTION_UP) {
            when (event.keyCode) {
                TURN_UP_KEY_CODE -> {
                    operateByPhysical(true)
                }
                TURN_DOWN_KEY_CODE -> {
                    operateByPhysical(false)
                }
            }
        }
        return super.dispatchKeyEvent(event)
    }

    private fun operateByPhysical(increase: Boolean) {
        val cookStep = viewModel.displayStep()
        val tabType = getSelectedTabType()
        if (tabType == CookSettingType.TEMP_SETTING) {
            cookStep?.uiData?.let {
                if (!it.isTempChange) {
                    return
                }
                if (increase) {
                    it.currentTemp++
                    if (it.currentTemp > it.limitMaxTemp) {
                        it.currentTemp = it.limitMaxTemp
                    }
                } else {
                    it.currentTemp--
                    if (it.currentTemp < it.limitMinTemp) {
                        it.currentTemp = it.limitMinTemp
                    }
                }
                viewModel.stepUiData.doingModify = true
                binding.controller.tempRingView.updateProgress(it.currentTemp)
                onUserChangeTemp(it.currentTemp)
            }
        } else if (tabType == CookSettingType.SPEED_SETTING) {
            cookStep?.uiData?.let {
                if (!it.isMotorGearChange) {
                    return
                }
                if (increase) {
                    it.currentSpeed++
                    if (it.currentSpeed > it.limitMaxSpeed) {
                        it.currentSpeed = it.limitMaxSpeed
                    }
                } else {
                    it.currentSpeed--
                    if (it.currentSpeed < it.limitMinSpeed) {
                        it.currentSpeed = it.limitMinSpeed
                    }
                }
                viewModel.stepUiData.doingModify = true
                binding.controller.speedRingView.updateProgress(it.currentSpeed)
                onUserChangeSpeed(it.currentSpeed)
            }
        }else if (tabType == CookSettingType.TIME_SETTING) {
            cookStep?.uiData?.let {
                if (!it.isTimeChange) {
                    return
                }
                dealWithTimeByOperation(it, increase)
            }
        }
    }

    private fun dealWithTimeByOperation(uiData: CookStepUiData, increase: Boolean) {
        //当前调节时间
        var targetTime = uiData.targetTime
        if (increase) {
            targetTime++
        }else {
            targetTime--
        }
        if (targetTime > uiData.limitMaxTime) {
            targetTime = uiData.limitMaxTime
        }
        if (targetTime < uiData.limitMinTime) {
            targetTime = uiData.limitMinTime
        }
        val sec = targetTime % 60
        val min = ((targetTime / 60) % 60 )
        val hour = (targetTime / 3600)
        var time = ""
        if(hour > 0){
            time += if(hour < 10) {
                "0${hour}"
            }else{
                "${hour}"
            }

            time += if(min < 10){
                ":0${min}"
            }else{
                ":${min}"
            }
        }else{
            time += if(min < 10){
                "0${min}"
            }else{
                "${min}"
            }

            time += if(sec < 10){
                ":0${sec}"
            }else{
                ":${sec}"
            }
        }
        uiData.isTimeChange = true
        uiData.targetTime = targetTime
        binding.controller.clSetTime.isTimeCanChange(true)
        // 回调到上面那个地方来设置时间
        TimeUtil.getTimes(targetTime, timeArray)
        binding.controller.clSetTime.setTimeInternal(
            timeArray[0],
            timeArray[1],
            timeArray[2],
            changeSetting = true,
            setByUser = true
        )
    }

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        if (keyCode == PRESS_DOWN_KEY_CODE) {
            Log.d("QQQQ", "catch key down")
            //当前是turbo模式下
            if (viewModel.cookingStep?.isTurboMode() == true) {
                turboDownEvent()
            } else {
                //非turbo模式
                if (viewModel.stepUiData.doingModify && runningState.toByte() != DevStatus.STOP) {
                    //用户调节中
                    viewModel.stepUiData.doingModify = false
                    clickFirstButton(BUTTON_TAG_CONFIRM)
                    Log.d("QQQQ", "click confirm")
                } else {
                    if (runningState == DevStatus.RUNNING.toInt()) {
                        viewModel.stepUiData.doingModify = false
                        clickFirstButton(BUTTON_TAG_STOP)
                        Log.d("QQQQ", "click stop")
                    } else if (runningState == DevStatus.STOP.toInt()) {
                        viewModel.stepUiData.doingModify = false
                        clickFirstButton(BUTTON_TAG_START)
                        Log.d("QQQQ", "click start")
                    } else {
                        Log.d("QQQQ", "nothing to do")
                    }
                }
            }
        }
        return super.onKeyDown(keyCode, event)
    }

    /**
     * 根据UI状态刷新View
     */
    @SuppressLint("SetTextI18n")
    private fun updateUiSetting(status: CookStepStatus) {
        val uiData = status.uiData
        if (viewModel.stepUiData.doingModify && uiData.runningStatus == DevStatus.RUNNING.toInt()) {
            binding.controller.btnStart.setVisible()
            binding.controller.btnReset.setVisible()
            binding.controller.btnStart.updateText(getString(R.string.confirm))
            binding.controller.btnReset.updateText(getString(R.string.cancel))
            binding.controller.btnStart.tag = BUTTON_TAG_CONFIRM
            binding.controller.btnReset.tag = BUTTON_TAG_CANCEL
        } else {
            if (uiData.runningStatus == DevStatus.STOP.toInt()) {
                binding.controller.btnStart.setVisible()
                binding.controller.btnReset.setVisible()
                binding.controller.btnStart.updateText(getString(R.string.start))
                binding.controller.btnReset.updateText("RESET")
                binding.controller.btnStart.tag = BUTTON_TAG_START
                binding.controller.btnReset.tag = BUTTON_TAG_RESET
            } else if (uiData.runningStatus == DevStatus.RUNNING.toInt()) {
                binding.controller.btnStart.setVisible()
                binding.controller.btnReset.setGone()
                binding.controller.btnStart.updateText(getString(R.string.pause))
                binding.controller.btnStart.tag = BUTTON_TAG_PAUSE
            } else {
                binding.controller.btnStart.setVisible()
                binding.controller.btnReset.setVisible()
                binding.controller.btnStart.updateText(getString(R.string.resume))
                binding.controller.btnReset.updateText(getString(R.string.stop))
                binding.controller.btnStart.tag = BUTTON_TAG_RESUME
                binding.controller.btnReset.tag = BUTTON_TAG_STOP
            }
        }
        // 已经开始运行, 并且不是当前步骤, 隐藏操作按钮, 禁止操作
        val modifyDisable = viewModel.isCookingStepStarted() && !viewModel.isCurrentOnCookingStep()
        binding.controller.speedRingView.setCanTouch(!modifyDisable && uiData.isMotorGearChange)
        binding.controller.tempRingView.setCanTouch(!modifyDisable && uiData.isTempChange)
        binding.controller.tvTurnLeft.isEnabled = !modifyDisable && uiData.isMotorDirectionChange
        binding.controller.tvTurnRight.isEnabled = !modifyDisable && uiData.isMotorDirectionChange
        binding.controller.clSetTime.isTimeCanChange(!modifyDisable && uiData.isTimeChange)
        if (modifyDisable || status.isManualStep()) {
            binding.controller.btnStart.setGone()
            binding.controller.btnReset.setGone()
        }

        binding.controller.speedRingView.setRange(
            uiData.limitMinSpeed.toFloat(), uiData.limitMaxSpeed.toFloat()
        )
        binding.controller.tempRingView.setRange(
            uiData.limitMinTemp.toFloat(), uiData.limitMaxTemp.toFloat()
        )
        binding.controller.clSetTime.setTimeRange(
            uiData.limitMinTime, uiData.limitMaxTime
        )

        // 播放电机转动动画
        if (viewModel.isCurrentOnCookingStep()
            && uiData.runningStatus == DevStatus.RUNNING.toInt()
        ) {
            playRotateAnimator(uiData.direction)
        } else {
            pauseRotateAnimator()
        }

        // 时间设置
        TimeUtil.getTimes(uiData.remainTime, timeArray)
        if (!viewModel.stepUiData.doingModify) {
            binding.controller.clSetTime.setTimeInternal(
                timeArray[0],
                timeArray[1],
                timeArray[2],
                true
            ) // 表盘时间设置-当前剩余时间
            TimeUtil.getTimes(uiData.targetTime, timeArray)
            binding.controller.tvCookingTimeTarget.updateText("--${TimeUtil.formatTime(timeArray)}--") // 下面的TAB-目标运行时间
        }
        TimeUtil.getTimes(uiData.remainTime, timeArray)
        binding.controller.tvSetTime.updateText(TimeUtil.formatTime(timeArray))                            // 下面的TAB-当前剩余时间

        // 温度
        if (!viewModel.stepUiData.doingModify) {
            binding.controller.tvCookingTempTarget.updateText("--${uiData.targetTemp}°C--")            // 下面的TAB-目标运行温度
            binding.controller.tempRingView.updateProgress(uiData.currentTemp)                                 // 圆环设置-当前运行温度
            binding.controller.tvRingTempText.updateText("${uiData.currentTemp}°C")                     // 圆环内的数字-当前运行温度
        }
        binding.controller.tvTempValue.updateText("${uiData.currentTemp}°C")                       // 下面的TAB-当前运行温度

        // 速度设置
        if (!viewModel.stepUiData.doingModify) {
            binding.controller.speedRingView.updateProgress(uiData.currentSpeed)                               // 圆环设置-当前运行速度
            binding.controller.tvSpeedText.updateText(uiData.currentSpeed.toString())                          // 圆环内的数字-当前运行速度
            binding.controller.tvSetSpeed.updateText(uiData.currentSpeed.toString())                           // 下面的TAB-当前运行速度
        }

        // 方向设置
        if (!viewModel.stepUiData.doingModify) {
            updateDirection(uiData.direction)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        audioUtil.releaseMedia()
    }

    private fun recordRecipes() {
        if (!isHistory) {
            isHistory = true
            FoodDataProvider.getUserDatabase().runInTransaction {
                FoodDataProvider.getUserDatabase().userInfoDao().insertHistoryRecipe(
                    UserHistoryRecipes(CURRENT_USER_ID, recipeNumber ?: "")
                )
            }
        }
    }
}